package org.ow2.jones.demo.booking.client;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.io.PrintWriter;
import java.net.URI;
import java.net.URLConnection;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;


/**
 * @author Samuel Quaireau
 */
public class BookingServlet extends HttpServlet {
	
	// For serialization
	// -----------------
	static final long serialVersionUID = 1;

	// ...
	// ---
    private static final javax.xml.namespace.QName SERVICE_NAME =
    	new javax.xml.namespace.QName("https://wiki.objectweb.org/ESBi", "BookingService");

	// Trace types
	// -----------
	public static final boolean CTE_TRC_ERROR         = true;
	public static final boolean CTE_TRC_DEBUG         = true;
	public static final boolean CTE_TRC_PARAM         = true;

	// Contantes
	// ---------
	public static final String CTE_ENCODING  = "iso-8859-1";
	public static final String CTE_SERVLET   = "";

	// Parameters
	// ----------
	public static final String CTE_PARAM_ACTION   = "action";

	// Parameter values
	// ----------------
	private static final String CTE_ACTION_EXIT        = "exit";
	private static final String CTE_ACTION_RESTART     = "restart";
	private static final String CTE_ACTION_EXECUTE     = "execute";

	// Constructor
	// -----------
	public BookingServlet() {
		super();
	}

	// Initializes this servlet
	// ------------------------
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
	}

	// Process incoming HTTP GET requests
	// ----------------------------------
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
		trace(CTE_TRC_DEBUG, "BookingServlet::doGet");
		performTask(request, response);
	}
	
	// Process incoming HTTP POST requests
	// -----------------------------------
	public void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
		trace(CTE_TRC_DEBUG, "BookingServlet::doPost");
		performTask(request, response);
	}

	// Process incoming requests for information
	// -----------------------------------------
	private void performTask(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {

		// Flux (XML ou HTML) & Output
		boolean isXml = false;
		String output = "";
		
		try {
			
			// Processing the "action parameter"
			String aAction = request.getParameter(CTE_PARAM_ACTION);
			trace(CTE_TRC_PARAM, "action: " + aAction);

			// Find the response content...
			String rootServletURL = "http://"+request.getServerName()+":"+request.getServerPort()+"/Booking/";
			if ((null == aAction) || (0 == aAction.length()) || aAction.startsWith(CTE_ACTION_RESTART)) {
				
				// Open the form...
				URLConnection urlConn = (new URI(rootServletURL + "xhtml/BookingClientForm.xhtml")).toURL().openConnection();
				InputStream inStream  = urlConn.getInputStream();

				// ...and read its content
				StringBuffer st = new StringBuffer();
				byte[] buf = new byte[8];
				int nb = 0;
				while (0 < (nb = inStream.read(buf)))
					st.append(new String(buf, 0, nb));
				output = st.toString();
				// trace(CTE_TRC_DEBUG, "BookingServlet : ...: " + st.toString());
				isXml = true;
				
			} else if (aAction.startsWith(CTE_ACTION_EXECUTE)) {
				
				// 1. Retrieving service parameters values
				int roomNumber = 0;
				boolean isRestaurant = false;
				DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
				DocumentBuilder        builder = factory.newDocumentBuilder();
				Document              document = builder.parse(new InputSource(request.getReader()));
				Node                  rootNode = document.getDocumentElement();
				NodeList              nodeList = rootNode.getChildNodes();
				for (int i=0 ; i<nodeList.getLength() ; i++) {
					Node currentNode = nodeList.item(i);
					if (currentNode.getNodeType() == Node.ELEMENT_NODE && currentNode.hasChildNodes() ) {
						NodeList nl = currentNode.getChildNodes();
						Node nlfc   = nl.item(0); // should either be the text of the node or a child element
						if (nlfc.getNodeType() == Node.TEXT_NODE) {
							String variable = currentNode.getNodeName();
							String varValue = nlfc.getNodeValue();
							trace(CTE_TRC_PARAM, " #token# ==> " + variable + " = " + varValue);
							if ("isRestaurant".equals(variable))
								isRestaurant = Boolean.parseBoolean(varValue);
							else if ("roomNumber".equals(variable))
								roomNumber = Integer.parseInt(varValue);
							else
								trace(CTE_TRC_DEBUG, "Unexpected parameter: " + variable + " = " + varValue);
						}
					}
				} 
				trace(CTE_TRC_PARAM, "Current values: " + isRestaurant + " & " + roomNumber);
				
				// 2. Calling the booking service
				boolean isResult = false;
				int bookingPrice = 0;
				boolean bookingResult = false;
				String returnedMessage = "";
		        try {
		        	org.ow2.jones.demo.booking.client.BookingSoapBindingStub binding = (org.ow2.jones.demo.booking.client.BookingSoapBindingStub)
		                          new org.ow2.jones.demo.booking.client.BookingServiceLocator(
		                        		  "http://s4allsdk.objectweb.org/demo/booking/booking.wsdl",
		                        		  SERVICE_NAME).getbooking();
		            // Time out after 20s
			        binding.setTimeout(20000);
			        // Test operation
			        javax.xml.rpc.holders.IntHolder iH     = new javax.xml.rpc.holders.IntHolder();
			        javax.xml.rpc.holders.BooleanHolder bH = new javax.xml.rpc.holders.BooleanHolder();
			        javax.xml.rpc.holders.StringHolder sH  = new javax.xml.rpc.holders.StringHolder();
			        binding.book(roomNumber, isRestaurant, iH, bH, sH);
		            isResult = true;
					bookingPrice = iH.value;
					bookingResult = bH.value;
					returnedMessage = sH.value;
		        } catch (javax.xml.rpc.ServiceException jre) {
		            jre.printStackTrace();
		            returnedMessage = jre.getMessage();
		        }
	            trace(CTE_TRC_PARAM, "RESULT: " + bookingPrice + ";" + bookingResult + ";" + returnedMessage);
//				int bookingPrice = 100;
//				boolean bookingResult = false;
//				String returnedMessage = "MSG";
//				isResult = true;
				
	            // 3. Gives the result
	            if (isResult) {
					try {
						String xmlLikeResult = "<result>" +
							"<bookingPrice>"+bookingPrice+"$</bookingPrice>" + 
							"<bookingResult>"+(bookingResult ? "Yes" : "No")+"</bookingResult> +" +
							"<returnedMessage>"+returnedMessage+"</returnedMessage>" +
							"</result>";
						ByteArrayOutputStream baos = new ByteArrayOutputStream();
						ByteArrayInputStream  bais = new ByteArrayInputStream(xmlLikeResult.getBytes());
						TransformerFactory tFactory = TransformerFactory.newInstance();
						Transformer transformer = tFactory.newTransformer(new StreamSource(rootServletURL + "xsl/bookingResult.xsl"));
						transformer.transform(new StreamSource(bais), new StreamResult(baos));
						baos.flush();
						output = new String(baos.toByteArray());
//						trace(CTE_TRC_DEBUG, "After XSL-Transformation: " + output);
						isXml = true;
					} catch (Exception e) {
						output = "<html><body>Unknown error: " + e.getMessage() + "</body></html>";
						isXml = false;
						e.printStackTrace();
					}
	            } else {
					output = "<html><body>Unknown error: " + returnedMessage + "</body></html>";
					isXml = false;
	            }
			} else if (aAction.startsWith(CTE_ACTION_EXIT)) {
				this.exit(request, response);
				return;
			} else {
				output = "<html>Unknown action: " + aAction + "<br><a href=\"" + CTE_SERVLET + "?action=exit\">Quitter</a></html>";
				isXml = false;
			}

			// ... and write it
			response.setContentType("text/" + (isXml ? "xml" : "html") + "; charset=" + CTE_ENCODING);
			response.setContentLength(output.length());
			PrintWriter out = response.getWriter();
			out.print(output);
			out.flush();
			out.close();

		} catch (Exception e) {
			trace(CTE_TRC_ERROR, "EXCEPTION: " + e.getMessage());
			e.printStackTrace();
			// Send an error msg
			output = "<html>Erreur de traitement de la page demandee: " + e.getMessage() + "</html>";
			response.setContentType("text/html; charset=" + CTE_ENCODING);
			response.setContentLength(output.length());
			java.io.PrintWriter out = response.getWriter();
			out.print(output);
			out.close();
		}

		trace(CTE_TRC_DEBUG, "BookingServlet::performTask OUT");
	}
	
	// Exit the booking service
	// ------------------------
	private void exit(HttpServletRequest request, HttpServletResponse response) throws ServletException, java.io.IOException {
		if (null != request) {
			HttpSession session = request.getSession();
			if (null != session)
				session.invalidate();
		}
		if (null != response)
			response.sendRedirect(CTE_SERVLET);
	}

	// Trace facilities
	// ----------------
	public static void trace(boolean displayType, String aTrace) {
		if (displayType) {
			System.out.println(aTrace);
		}
		//if (null != logger) {
		//	logger.debug(aTrace);
		//}
	}

}
